<?php

namespace Drupal\scribit_pro;

use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class HelperService.
 *
 * Helper service that provides additional functionality we don't want to
 * include in the api service.
 */
class HelperService {
  use StringTranslationTrait;

  /**
   * YouTube video url to be used for posting video's to Scribit Pro.
   */
  private const YOUTUBE_VIDEO_URL = 'https://www.youtube.com/watch?v=';

  /**
   * Vimeo video url to be used for posting video's to Scribit Pro.
   */
  private const VIMEO_VIDEO_URL = 'https://vimeo.com/';

  /**
   * Drupal\scribit_pro\ApiService definition.
   *
   * @var \Drupal\scribit_pro\ApiService
   */
  protected $api;

  /**
   * Constructs a new HelperService object.
   */
  public function __construct(ApiService $scribit_pro_api) {
    $this->api = $scribit_pro_api;
  }

  /**
   * Get local Scribit ID.
   *
   * Get locally stored Scribit ID.
   *
   * @return string
   *   Returns the locally stored Scribit ID.
   */
  public function getScribitId() {
    return $this->api->getScribitId();
  }

  /**
   * Submits a professional services request for a given video.
   *
   * @param string $video_url
   *   URL of the video to be processed.
   * @param string
   *   $remarks Additional remarks or comments for the request.
   * @param array $services
   *   List of requested services.
   * @param bool $urgent
   *   Flag to indicate if the request is urgent.
   * @param string $voice_gender
   *   Preferred gender for voice-over.
   * @param string $voice_language
   *   Preferred language for voice-over.
   *
   * @return mixed
   *   The response from the API after submitting the request.
   */
  public function postProServicesRequest($video_url, $remarks, $services, $urgent, $voice_gender, $voice_language)
  {
    if (!empty($video_url) && $video_id = $this->extractYouTubeIdFromUrl($video_url)) {
      return $this->api->postProServicesRequest($this->prepareYouTubeVideoUrl($video_id), $video_id, $remarks, $services, $urgent, $voice_gender, $voice_language);
    }
    else if (!empty($video_url) && $video_id = $this->extractVimeoIdFromUrl($video_url)) {
      return $this->api->postProServicesRequest($this->prepareVimeoVideoUrl($video_id), $video_id, $remarks, $services, $urgent, $voice_gender, $voice_language);
    }
  }

  /**
   * Prepare YouTube video url.
   *
   * @param string $video_id
   *   YouTube Video id.
   *
   * @return string
   *   Returns complete YouTube video url for Scribit Pro.
   */
  public function prepareYouTubeVideoUrl($video_id) {
    return self::YOUTUBE_VIDEO_URL . $video_id;
  }

  /**
   * Prepare Vimeo video url.
   *
   * @param string $video_id
   *   Vimeo Video id.
   *
   * @return string
   *   Returns complete Vimeo video url for Scribit Pro.
   */
  public function prepareVimeoVideoUrl($video_id) {
    return self::VIMEO_VIDEO_URL . $video_id;
  }

  /**
   * Get YouTube id from video url.
   *
   * @param string $url
   *   YouTube url the video id needs exctracted from.
   *
   * @return string|null
   *   YouTube id.
   */
  public function extractYouTubeIdFromUrl($url) {
    preg_match('/(?<=(?:v|i)=)[a-zA-Z0-9-]+(?=&)|(?<=(?:v|i)\/)[^&\n]+|(?<=embed\/)[^"&\n]+|(?<=(?:v|i)=)[^&\n]+|(?<=youtu.be\/)[^&\n]+/', $url, $id);
    if (empty($id)) {
      return NULL;
    }
    return $id[0] ? $id[0] : NULL;
  }

  /**
   * Get YouTube id from video url.
   *
   * @param string $url
   *   YouTube url the video id needs exctracted from.
   *
   * @return string|null
   *   YouTube id.
   */
  public function extractVimeoIdFromUrl($url) {
    preg_match('/(https?:\/\/)?(www\.)?(player\.)?vimeo\.com\/([a-z]*\/)*([0-9]{6,11})[?]?.*/', $url, $id);
    if (empty($id)) {
      return NULL;
    }
    return $id[5] ? $id[5] : NULL;
  }

  /**
   * Get available services for Scribit Pro.
   *
   * @return array
   *   Array containing available Scribit Pro Services.
   */
  public function getServices() {
    return [
      'audio description' => $this->t('Audio description'),
      'subtitles' => $this->t('Subtitles'),
      'transcript' => $this->t('Transcript'),
      'sign language' => $this->t('Sign language'),
    ];
  }

  /**
   * Get available languages for Scribit Pro.
   *
   * @return array
   *   Array containing available Scribit Pro Languages.
   */
  public function getLanguages() {
    return [
      'en_uk' => $this->t('English (EN)'),
      'nl_nl' => $this->t('Dutch (NL)'),
      'nl_be' => $this->t('Flemish (BE)'),
      'sv_se' => $this->t('Swedish (SE)'),
      'de_de' => $this->t('German (DE)'),
      'it_it' => $this->t('Italian (IT)'),
      'pl_pl' => $this->t('Polish (PL)'),
      'fr_fr' => $this->t('French (FR)'),
      'es_es' => $this->t('Spanish (ES)'),
    ];
  }

  /**
   * Get available voices for Scribit Pro.
   *
   * @return array
   *   Multi dimensional array containing available Scribit Pro Voices per
   *   available language code.
   */
  public function getVoices() {
    return [
      'nl_nl' => [
        'male' => 'Guus',
        'female' => 'Ilse',
      ],
      'en_uk' => [
        'male' => 'Hugh',
        'female' => 'Alice',
      ],
      'nl_be' => [
        'female' => 'Veerle',
      ],
      'sv_se' => [
        'male' => 'Sven',
        'female' => 'Karin',
      ],
      'de_de' => [
        'male' => 'Max',
        'female' => 'Lena',
      ],
      'it_it' => [
        'female' => 'Gina',
      ],
      'pl_pl' => [
        'female' => 'Ania',
      ],
      'fr_fr' => [
        'male' => 'Benoit',
        'female' => 'Elise',
      ],
      'es_es' => [
        'female' => 'Lola',
      ],
    ];
  }

  /**
   * Get Voice key.
   *
   * Returns the voice key for the given language and gender.
   *
   * @param string $language
   *   Language code.
   * @param string $gender
   *   Gender (male or female).
   *
   * @return string
   *   Voice key for given language and gender.
   */
  public function getVoiceKey($language, $gender) {
    return $this->getVoices()[$language][$gender];
  }

  /**
   * Check if Scribit Pro credentials have been set.
   *
   * @return bool
   *   Indicating whether credentials have been set.
   */
  public function hasCredentials() {
    return $this->api->hasCredentials();
  }

  /**
   * Get pro services request for given video url.
   *
   * @param string $video_url
   *   Video url.
   *
   * @return string|null
   *   Returns pro services request id if found, otherwise null.
   */
  public function getProServicesRequest($video_url) {
    if (empty($video_url)) {
      return NULL;
    }
    if ($video_id = $this->extractYouTubeIdFromUrl($video_url)) {
      return $this->api->getProServicesRequest('YouTube', $video_id);
    }
    else if ($video_id = $this->extractVimeoIdFromUrl($video_url)) {
      return $this->api->getProServicesRequest('Vimeo', $video_id);
    }

    return NULL;
  }

  /**
   * Get media links for given video url.
   *
   * @param string $video_url
   *   Video url.
   *
   * @return array|null
   *   Returns media links if found, otherwise null.
   */
  public function getMediaLinks($video_url) {
    if (empty($video_url)) {
      return NULL;
    }
    if ($video_id = $this->extractYouTubeIdFromUrl($video_url)) {
      return $this->api->getMediaLinks('YouTube', $video_id);
    }
    else if ($video_id = $this->extractVimeoIdFromUrl($video_url)) {
      return $this->api->getMediaLinks('Vimeo', $video_id);
    }
    return NULL;
  }

}
