<?php

namespace Drupal\scribit_pro\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\StringTextfieldWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\scribit_pro\HelperService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'scribit_pro_widget' widget.
 *
 * @FieldWidget(
 *   id = "scribit_pro_widget",
 *   module = "scribit_pro",
 *   label = @Translation("Scribit Pro oEmbed URL"),
 *   field_types = {
 *     "string",
 *   },
 * )
 */
class ScribitProWidget extends StringTextfieldWidget {

  /**
   * Scribit Pro helper service.
   *
   * @var \Drupal\scribit_pro\HelperService
   */
  protected $helper;

  /**
   * Constructs a WidgetBase object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\scribit_pro\HelperService $helper
   *   Scribit Pro helper service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, HelperService $helper) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->helper = $helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('scribit_pro.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $element['scribit_pro'] = [
      '#tree' => TRUE,
      '#type' => 'fieldset',
      '#title' => $this->t('Scribit Pro'),
      '#collapsible' => FALSE,
      '#required' => FALSE,
      '#attributes' => [
        'data-scribit-pro-container' => 1,
      ],
    ];

    $element['scribit_pro']['#attached']['library'][] = 'scribit_pro/field_widget';

    if (isset($element['value']['#default_value']) && $pro_services_request = $this->helper->getProServicesRequest($element['value']['#default_value'])) {
      $element['scribit_pro']['video_exists_message'] = [
        '#type' => 'item',
        '#markup' => $this->t('This video has already been submitted to Scribit Pro'),
      ];
      $element['scribit_pro']['details'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Details'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      ];
      $translated_services = [];
      foreach ($pro_services_request['services'] as $service) {
        $translated_services[] = $this->helper->getServices()[$service] ?? $this->t('Unknown service');
      }
      $report = [
        'status' => [ 'label' => $this->t('Status'), 'value' => $pro_services_request['status'] ],
        'services' => [ 'label' => $this->t('Services'), 'value' => implode(', ', $translated_services) ],
        'urgent' => [ 'label' => $this->t('Urgent'), 'value' => $pro_services_request['urgent'] ? $this->t('Yes') : $this->t('No') ],
        'remarks' => [ 'label' => $this->t('Remarks'), 'value' => $pro_services_request['remarks'] ],
        'estimated_delivery_date' => [ 'label' => $this->t('Estimated delivery date'), 'value' => $pro_services_request['estimated_delivery_date'] ?? $this->t('Unknown') ],
      ];
      foreach ($report as $key => $value) {
        $element['scribit_pro']['details'][$key] = $this->createSystemReportRow($value['label'], $value['value']);
      }
    }
    elseif (isset($element['value']['#default_value']) && $this->helper->getMediaLinks($element['value']['#default_value'])) {
      $element['scribit_pro']['video_exists_message'] = [
        '#type' => 'item',
        '#markup' => $this->t('This video has already been submitted to Scribit Pro'),
      ];
    }
    else {
      $element['scribit_pro']['enable'] = [
        '#type' => 'checkbox',
        '#tree' => TRUE,
        '#title' => $this->t('Submit video to Scribit Pro'),
        '#attributes' => [
          'data-scribit-pro-enable' => 1,
        ],
        '#required' => FALSE,
      ];
      $element['scribit_pro']['services'] = [
        '#type' => 'checkboxes',
        '#tree' => TRUE,
        '#title' => $this->t('Services'),
        '#options' => $this->helper->getServices(),
        '#attributes' => [
          'data-scribit-pro-field' => 'services',
        ],
        '#states' => [
          'visible' => [
            ':input[data-scribit-pro-enable="1"]' => ['checked' => TRUE],
          ],
        ],
        '#element_validate' => [
          [$this, 'settingsFormRequiredValidate'],
        ],
        '#required' => FALSE,
        '#description' => $this->t('Select the services you want to be added to the video.'),
      ];

      $element['scribit_pro']['voice'] = [
        '#type' => 'radios',
        '#tree' => TRUE,
        '#title' => $this->t('Voice'),
        '#options' => [
          'male' => $this->t('Male'),
          'female' => $this->t('Female'),
        ],
        '#attributes' => [
          'data-scribit-pro-field' => 'voice',
        ],
        '#states' => [
          'visible' => [
            ':input[data-scribit-pro-enable="1"]' => ['checked' => TRUE],
          ],
        ],
        '#element_validate' => [
          [$this, 'settingsFormRequiredValidate'],
        ],
        '#required' => FALSE,
        '#description' => $this->t('Select the voice you want to be used for the audio description.'),
      ];

      $element['scribit_pro']['language'] = [
        '#type' => 'radios',
        '#tree' => TRUE,
        '#title' => $this->t('Language'),
        '#options' => $this->helper->getLanguages(),
        '#attributes' => [
          'data-scribit-pro-field' => 'language',
        ],
        '#states' => [
          'visible' => [
            ':input[data-scribit-pro-enable="1"]' => ['checked' => TRUE],
          ],
        ],
        '#element_validate' => [
          [$this, 'settingsFormRequiredValidate'],
        ],
        '#description' => $this->t('Select the language you want to be used for the audio description.'),
      ];
      $element['scribit_pro']['urgent'] = [
        '#type' => 'checkbox',
        '#tree' => TRUE,
        '#title' => $this->t('Urgent'),
        '#attributes' => [
          'data-scribit-pro-field' => 'urgent',
        ],
        '#states' => [
          'visible' => [
            ':input[data-scribit-pro-enable="1"]' => ['checked' => TRUE],
          ],
        ],
        '#required' => FALSE,
        '#description' => $this->t('This will prioritize the processing of your video. An extra charge may apply.'),
      ];
      $element['scribit_pro']['remarks'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Remarks'),
        '#description' => $this->t('Add any remarks you want to share with the Scribit Pro team.'),
        '#states' => [
          'visible' => [
            ':input[data-scribit-pro-enable="1"]' => ['checked' => TRUE],
          ],
        ],
        '#required' => FALSE,
      ];
    }

    /** @var \Drupal\media\Plugin\media\Source\OEmbedInterface $source */
    $source = $items->getEntity()->getSource();
    $providers = $source->getProviders();

    $description[] = $this->t('You can link to media from the following services: @providers', ['@providers' => implode(', ', $providers)]);

    if (!empty(array_diff($providers, ['YouTube', 'Vimeo']))) {
      $description[] = $this->t('Be aware that at the moment only YouTube and Vimeo videos can be submitted to Scribit Pro.');
    }

    if (!empty($element['#value']['#description'])) {
      $description[] = $element['#value']['#description'];
    }

    $element['value']['#description'] = count($description) > 1 ? [
      '#theme' => 'item_list',
      '#items' => $description,
    ] : reset($description);

    return $element;
  }

  public function createSystemReportRow($label, $value) {
    return [
      '#type' => 'container',
      '#attributes' => ['class' => ['system-status-report__row']],
      'label' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $label,
        '#attributes' => ['class' => ['system-status-report__status-title']],
      ],
      'content' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $value,
        '#attributes' => ['class' => ['system-status-report__entry__value']],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateElements(array &$form, FormStateInterface $form_state) {
    parent::validateElements($form, $form_state);

    $plugin_id = $this->getPluginId();
    $assurance_levels_enabled = $form_state->getValue([
      $plugin_id,
      'assurance_levels_enabled',
    ]);
    // If the authentication provider is enabled,
    // assurance_levels_enabled is required!
    if ($form_state->isValueEmpty([$plugin_id, 'enabled']) === FALSE) {
      $checked = array_filter($assurance_levels_enabled);
      if (empty($checked)) {
        $form_state->setError($form[$plugin_id]['assurance_levels_enabled'], $this->t('@name field is required.', [
          '@name' => $form[$plugin_id]['assurance_levels_enabled']['#title'],
        ]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsFormRequiredValidate($element, FormStateInterface $form_state) {
    $scribit_pro_enabled = $element['#parents'];
    array_pop($scribit_pro_enabled);
    $scribit_pro_enabled[] = 'enable';
    $scribit_pro_enabled_value = $form_state->getValue($scribit_pro_enabled);
    $submitted_value = $form_state->getValue($element['#parents']);
    if ($element['#type'] == 'checkboxes') {
      $submitted_value = array_filter($submitted_value);
    }
    if ($scribit_pro_enabled_value && empty($submitted_value)) {
      $form_state->setError($element, $this->t('@name field is required.', ['@name' => $element['#title']]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    // Check if the form has been submitted.
    if ($form_state->isValidationComplete()) {
      $prepared_values = [];
      foreach ($values as $value_key => $item) {
        foreach ($item as $item_key => $item_value) {
          if ($item_key == 'scribit_pro') {
            if (isset($item_value['enable']) && $item_value['enable'] == 1 && !empty($item_value['voice']) && !empty($item_value['language']) && !empty(array_filter($item_value['services'])) && !$this->helper->getProServicesRequest($item['value'])) {
              $post_video = $this->helper->postProServicesRequest($item['value'], $item_value['remarks'] ?? '', array_keys(array_filter($item_value['services'])), (bool)$item_value['urgent'] ?? FALSE, $item_value['voice'], $item_value['language']);
              if (!isset($post_video['error']) || !$post_video['error']) {
                \Drupal::messenger()->addStatus('Video has been submitted to Scribit Pro');
              }
            }
          } else {
            $prepared_values[$value_key][$item_key] = $item_value;
          }
        }
      }
      return parent::massageFormValues($prepared_values, $form, $form_state);
    }
    return parent::massageFormValues($values, $form, $form_state);
  }

}
